function [C, Coh] = larc(D, X, varargin)
%LARC Performs LARC coding of X in D.
%
%INPUT
%D     Dictionary (dxl)
%X     Data matrix (dxn)
%mut   Residual coherence stopping criterion (default 0)
%k     Coding cardinality stopping criterion (default k=min(d-1,l))
%err   Residual 2-norm stopping criterion (default 1e-6)
%v     Verbosity (optional, v=0 default, v=1 verbose)
%
%Pass arguments mut, k, err and v as argument/value pairs, see example call.
%
%OUTPUT
%C     Coding of data X in dictionary D (lxn)
%Coh   TODO
%
%EXAMPLE CALL
%> [C, Coh] = larc(D, X, 'mut', 0.2, 'v', 1);

%% Parameters & Checks

[mut, k, err, v] = process_options(varargin, 'mut', 0 , 'k', inf , 'err', 1e-6, 'v', 0);

% Dimensions
[d1,l] = size(D);
[d2,n] = size(X);

if (d1 ~= d2)
    error('Feature space dimensions of D and X must agree.');
else
    d = d1;
end

% TODO: some check on n?

% TODO Parameter checks for mut, k, err

% Cardinality stopping criterion
if (k > min(d-1,l)) % TODO check
    k = min(d-1,l);
end

% Verbose Parameter
if (v ~= 0 && v ~= 1)
    error('Invalid value for argument v.');
end

%% LARC

[C, Coh] = larcmex(D, X, D'*X, D'*D, mut, k, err, v);

end
