/*
 * larcmex.c
 */

#include "mex.h"
#include "larccore.h"

/* Input Arguments */

#define IN_D		prhs[0]
#define IN_X		prhs[1]
#define IN_DtX		prhs[2]
#define IN_G		prhs[3]
#define IN_MUT		prhs[4]
#define IN_K		prhs[5]
#define IN_ERR		prhs[6]
#define IN_VERBOSE	prhs[7]

/* Output Arguments */

#define	OUT_C		plhs[0]
#define	OUT_Coh		plhs[1]

/*
 * Mex Gateway Function
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	double *D, *X, *DtX, *G;
	double mut, err;
	mwSize k;
	int verbose;
	mwSize d, l, n;
	mxArray *C, *Coh;
	double *C_data, *Coh_data;

	/* Retrieve Inputs */
	/* TODO use mxIsDouble to check array type */
	D = mxGetPr(IN_D);
	X = mxGetPr(IN_X);
	DtX = mxGetPr(IN_DtX);
	G = mxGetPr(IN_G);
	mut = mxGetScalar(IN_MUT);
	k = (mwSize)mxGetScalar(IN_K);
	err = mxGetScalar(IN_ERR);
	verbose = (int)mxGetScalar(IN_VERBOSE);

	if (verbose >= 1) {
		mexPrintf("mut=%f\n", mut);
		mexPrintf("k=%d\n", k);
		mexPrintf("err=%f\n", err);
		mexPrintf("verbose=%d\n", verbose);
	}

	/* Matrix Dimensions */
	d = mxGetM(IN_D);
	l = mxGetN(IN_D);
	n = mxGetN(IN_X);

	/* Prepare Outputs */
  C = mxCreateDoubleMatrix(l, n, mxREAL); /* Sparse coding matrix */
	C_data = mxGetPr(C); /* Pointer to data in C */

	Coh = mxCreateDoubleMatrix(k, n, mxREAL); /* Residual coherence curve matrix */
	Coh_data = mxGetPr(Coh); /* Pointer to data in Coh */

	/* LARC Coding */

	larccore(C_data, Coh_data, D, X, DtX, G, mut, k, err, verbose, d, l, n);

	/* Output */

	OUT_C = C;
	OUT_Coh = Coh;

	return;
}
